#ifndef common_binaries_h
#define common_binaries_h

#include "objects_types.h"

#ifdef _MSC_VER
#pragma pack(push, 1)
#define GCC_PACKED 
#else 

#define GCC_PACKED __attribute__((packed))

#endif

//
// Binary file structure
namespace binary
{
	//
	// File header
	struct header
	{
		int signature;   
		int version;     
		unsigned int objectTableOffset;
		unsigned int objectTableBytesSize;
	} GCC_PACKED;

	// Rigth values for signature and version
	const int signature = 'schm';
	const int version = 0x00010001;

	//
	// Record in Object Table
	struct record
	{
		short int type;   // See objects_types.h
		short int flags;
		int estimate;
		unsigned int offset;
		unsigned int size;

		// Flags
		enum
		{
			named = 0x01,
		};
	} GCC_PACKED;

	//
	// Link to object
	union link
	{
		struct
		{
			unsigned item  : 30;  // Object id
			unsigned table : 2;   // Table id
		} l;

		int value;

		// Tables
		enum
		{
			standard = 0,
			program = 1,
			local = 2
		};
	} GCC_PACKED;

	/////////
	//
	// literals
	//
	/////////

	//
	// numeric
	struct numeric
	{
		double value;
	} GCC_PACKED;

	//
	// character
	struct character
	{
		char value;
	} GCC_PACKED;

	//
	// boolean
	struct boolean
	{
		char value;
	} GCC_PACKED;

	// 
	// string
	//struct string
	//{
	//   char value[anysize]; // string size specified in ObjectRecord
	//};

	// 
	// symbol
	//struct symbol
	//{
	//   char value[anysize]; // symbol size specified in ObjectRecord
	//};

	//
	// pair
	struct pair
	{
		link car;
		link cdr;
	} GCC_PACKED;

	//
	// empty_list
	//struct empty_list
	//{
	//};

	///////////
	//
	// Code
	//
	///////////

	//
	// lambda
	struct lambda
	{
		// arguments count
		struct
		{
			unsigned short count;  // Count of usual arguments
			short list;   // List argument: to use, or not to use?
		} argc;

		// expression
		link exp;
	} GCC_PACKED;

	//
	// call
	struct call
	{
		unsigned short argc;
		link ops[1]; // operator and operands, anysize array
	} GCC_PACKED;

	//
	// cond
	struct cond
	{
		unsigned short count;
		link clause[1]; // tests and expressions, anysize array
	} GCC_PACKED;

	//
	// if expression
	struct if_
	{
		link condition;
		link consequence;
		link alternate;
	} GCC_PACKED;

	//
	// Or expression
	struct or_
	{
		unsigned short count;
		link test[1]; // tests, anysize array
	} GCC_PACKED;

	//
	// And expression
	struct and_
	{
		unsigned short count;
		link test[1]; // tests, anysize array
	} GCC_PACKED;
}

#ifdef _MSC_VER
#pragma pack(pop)
#endif

#endif
