#ifndef compiler_expression_expression_code_h
#define compiler_expression_expression_code_h

#include "expression_base.h"
#include "../utils/pointer_vector.h"

//
// I dont want include these files,
// becouse code does not use them
//
//#include "../streams/stream_input.h"
//#include "../lexer/lexer.h"

namespace lexer
{
	class token;
}

namespace stream
{
	template<typename T>
	class input;
}

namespace expression
{
	//
	// Call expression
	class call : public base
	{
	public:

		// arranged in ./syntacter/syntacter_code.cpp
		static call* read(stream::input<lexer::token>& i);

		virtual ~call();

		virtual binary::link link(linker::context& c);
		virtual int estimate(linker::context& c);

	private:

		call(base* operation);
		void add(base* e);

	private:

		base* m_operation;
		pointer_vector<base> m_operands;
	};

	//
	// if expression
	class if_ : public base
	{
	public:

		// arranged in ./syntacter/syntacter_code.cpp
		static if_* read(stream::input<lexer::token>& i);
		
		virtual ~if_();

		virtual binary::link link(linker::context& c);
		virtual int estimate(linker::context& c);

	private:

		if_(base* condition, base* cons, base* alt);

	private:

		base* m_condition;
		base* m_consequence;
		base* m_alternate;
	};

	//
	// cond expression
	class cond : public base
	{
	public:

		// arranged in ./syntacter/syntacter_code.cpp
		static cond* read(stream::input<lexer::token>& i);
				
		virtual ~cond();

		virtual binary::link link(linker::context& c);
		virtual int estimate(linker::context& c);

	private:

		cond();
		void addClause(base* c, base* e);
		void addElse(base* e);

	private:

		pointer_vector<base> m_test;
		pointer_vector<base> m_exp;
		base* m_else;
	};

	//
	// or expression
	class or_ : public base
	{
	public:

		// arranged in ./syntacter/syntacter_code.cpp
		static or_* read(stream::input<lexer::token>& i);

		virtual binary::link link(linker::context& c);
		virtual int estimate(linker::context& c);

	private:

		or_()
		{
		}

		void add(base* t);

	private:

		pointer_vector<base> m_test;
	};

	//
	// and expression
	class and_ : public base
	{
	public:

		// arranged in ./syntacter/syntacter_code.cpp
		static and_* read(stream::input<lexer::token>& i);

		virtual binary::link link(linker::context& c);
		virtual int estimate(linker::context& c);

	private:

		and_()
		{
		}

		void add(base* t);

	private:

		pointer_vector<base> m_test;
	};
}

#endif
