#include "expression_lambda.h"
#include <stdexcept>

namespace expression
{
	//
	// Function parameter. Need for correct 
	// and simple estimation
	class parameter : public base
	{
	public:
		
		virtual binary::link link(linker::context& c)
		{
			throw std::runtime_error("expression::parameter::link: somebody call me!");
		}
	};

	//
	// Local function context
	class local_context : public linker::context
	{
	public:

		static base* getParameter()
		{
			static parameter p;
			return &p;
		}

		local_context(linker::context& c, const name2index& defs) 
					: m_parent(c), m_definitions(defs), m_args(0)
		{
		}

		local_context(linker::context& c, const name2index& defs, base** args) 
					: m_parent(c), m_definitions(defs), m_args(args)
		{
		}

		virtual expression::base* getExpr(const std::string& name) const
		{
			int index = m_definitions[name];

			if (index >= 0)
			{
				if (m_args)
				{
					return const_cast<base*>(m_args[index]);
				}
				else
				{
					return getParameter();
				}
			}

			return m_parent.getExpr(name);
		}

		virtual binary::link getLink(const std::string& name) const
		{
			int index = m_definitions[name];

			if (index >= 0)
			{
				binary::link l;
				l.l.table = binary::link::local;
				l.l.item = index;

				return l;
			}

			return m_parent.getLink(name);
		}

		virtual binary::link add(linker::object* obj)
		{
			return m_parent.add(obj);
		}

	private:
	
		const name2index& m_definitions;
		base** m_args;
		linker::context& m_parent;
	};

	//
	// Set and clear flag m_isEstimatedNow
	class set_and_clear
	{
	public:

		set_and_clear(bool& b) : m_b(b)
		{
			m_b = true;
		}

		~set_and_clear()
		{
			m_b = false;
		}

	private:

		bool& m_b;
	};
}

////////////////////////////////////
//
// Lambda expression
//
////////////////////////////////////

expression::lambda::lambda()
{
	m_function = 0;
	m_lastArgumentIsList = false;
	m_isEstimatedNow = false;
}

expression::lambda::~lambda()
{
	delete m_function;
}

binary::link expression::lambda::link(linker::context& c)
{
	local_context lc(c, m_args);

	return c.add(linker::object::create_lambda(
									m_args.size(), 
									m_lastArgumentIsList, 
									m_function->link(lc)));
}

int expression::lambda::estimateApply(linker::context& c, size_t argc, base** args)
{
	if (m_isEstimatedNow)
	{
		// recursion!
		return (0x01 << 16);
	}

	local_context lc(c, m_args, args);
	set_and_clear sac(m_isEstimatedNow);

	return m_function->estimate(lc);
}

void expression::lambda::addArgument(const std::string& a)
{
	m_args.add(a, (int)m_args.size());
}

void expression::lambda::setListArgument(const std::string& a)
{
	m_args.add(a, (int)m_args.size());
	m_lastArgumentIsList = true;
}

void expression::lambda::setFunction(base* e)
{
	m_function = e;
}