#ifndef compiler_expression_expression_literals_h
#define compiler_expression_expression_literals_h

#include "expression_base.h"
#include "../linker/linker_object.h"

#include <string>

namespace expression
{
	//
	// Boolean constant
	class boolean : public base
	{
	public:

		boolean(bool v) : m_value(v)
		{
		}

		virtual binary::link link(linker::context& c)
		{
			return c.add(linker::object::create(m_value));
		}

	private:

		const bool m_value;
	};

	//
	// Character constant
	class character : public base
	{
	public:

		character(char v) : m_value(v)
		{
		}

		virtual binary::link link(linker::context& c)
		{
			return c.add(linker::object::create(m_value));
		}

	private:

		const char m_value;
	};

	//
	// String constant
	class string : public base
	{
	public:

		string(const std::string& v) : m_value(v)
		{
		}

		virtual binary::link link(linker::context& c)
		{
			return c.add(linker::object::create_string(m_value));
		}

	private:

		const std::string m_value;
	};

	//
	// Numeric constant
	class numeric : public base
	{
	public:

		numeric(double v) : m_value(v)
		{
		}

		virtual binary::link link(linker::context& c)
		{
			return c.add(linker::object::create(m_value));
		}

	private:

		const double m_value;
	};

	//
	// Some symbol
	class symbol : public base
	{
	public:

		symbol(const std::string& v) : m_value(v)
		{
		}

		virtual binary::link link(linker::context& c)
		{
			return c.add(linker::object::create_symbol(m_value));
		}

	private:

		const std::string m_value;
	};

	//
	// Pair
	class pair : public base
	{
	public:

		pair(base* car, base* cdr) : m_car(car), m_cdr(cdr)
		{
		}

		virtual ~pair()
		{
			delete m_car;
			delete m_cdr;
		}

		virtual binary::link link(linker::context& c)
		{
			return c.add(linker::object::create_pair(estimate(c), m_car->link(c), m_cdr->link(c)));
		}

		virtual int estimate(linker::context& c)
		{
			return base::estimate(c) + m_car->estimate(c) + m_cdr->estimate(c);
		}

	private:

		base* m_car;
		base* m_cdr;
	};

	//
	// empty list
	class empty_list : public base
	{
	public:

		empty_list() 
		{
		}

		virtual binary::link link(linker::context& c)
		{
			return c.add(linker::object::create_empty_list());
		}
	};
}

#endif
