#include "lexer.h"

//////////////////////
//
// Constructors
//
//////////////////////

lexer::token::token() 
{
	m_type = null;
}

lexer::token::token(tp t) : m_type(t)
{
	m_numeric = 0;
	m_boolean = false;
	m_character = 0;
}

lexer::token::token(tp t, const std::string& str) : m_type(t), m_str(str)
{
	m_numeric = 0;
	m_boolean = false;
	m_character = 0;
}

lexer::token::token(double value) : m_type(numeric), m_numeric(value)
{
	m_boolean = false;
	m_character = 0;
}

lexer::token::token(bool value) : m_type(boolean), m_boolean(value)
{
	m_numeric = 0;
	m_character = 0;
}

lexer::token::token(char value) : m_type(character), m_character(value)
{
	m_numeric = 0;
	m_boolean = false;
}

lexer::token::token(const token& t) : m_type(t.m_type), m_str(t.m_str)
{
	m_numeric = t.m_numeric;
	m_boolean = t.m_boolean;
	m_character = t.m_character;
}

lexer::token& lexer::token::operator=(const token& t)
{
	m_type = t.m_type;
	m_str = t.m_str;
	m_numeric = t.m_numeric;
	m_boolean = t.m_boolean;
	m_character = t.m_character;
	return *this;
}

///////////////////////
//
// Classificators
//
///////////////////////

lexer::token::tp lexer::token::type() const
{
	return m_type;
}

bool lexer::token::isLiteral() const
{
	return ((m_type == character) 
		||  (m_type == string) 
		||  (m_type == numeric) 
		||  (m_type == boolean));
}

bool lexer::token::isIdentifier() const
{
	return ((m_type == lambda) 
		||  (m_type == if_) 
		||  (m_type == cond) 
		||  (m_type == and_) 
		||  (m_type == or_) 
		||  (m_type == define) 
		||  (m_type == else_)
		||  (m_type == variable));
}

bool lexer::token::isVariable() const
{
	return (m_type == variable);
}

///////////////////////
//
// Value
//
///////////////////////

const std::string& lexer::token::getString() const
{
	return m_str;
}

double lexer::token::getNumeric() const
{
	return m_numeric;
}

bool lexer::token::getBoolean() const
{
	return m_boolean;
}

char lexer::token::getCharacter() const
{
	return m_character;
}


