#ifndef compiler_streams_stream_input_h
#define compiler_streams_stream_input_h

#include <stack>
#include <stdexcept>

namespace stream
{
	// , ,   
	class eof : public std::runtime_error
	{
	public:

		eof() : std::runtime_error("eof")
		{
		}
	};

	//
	//   " "
	//   ,   eof
	//    -   eof
	template <class T>
	class source
	{
	public:

		source() : m_eofReached(false)
		{
		}

		virtual T get() = 0;

	protected:

		bool m_eofReached;
	};

	//
	//     
	template <class T>
	class input
	{
	public:

		input(source<T>& s) : m_source(s)
		{
		}

		T get()
		{
			if (!m_last.empty())
			{
				T t = m_last.top();
				m_last.pop();
				return t;
			}

			return m_source.get();
		}

		void putback(const T& t)
		{
			m_last.push(t);
		}

	private:

		std::stack<T> m_last;
		source<T>& m_source;
	};
}

#endif
