#ifndef compiler_syntacter_syntacter_h
#define compiler_syntacter_syntacter_h

#include "syntacter_expression.h"

#include "../streams/stream_input.h"
#include "../lexer/lexer.h"

#include "../utils/pointer_vector.h"

#include <string>

namespace syntacter
{
	/*
	<program> ::= <definition>* <expression>
	<definition> ::= (define <variable> <expression>)
	*/

	//
	// Array of definitions
	class definitions
	{
	public:

		size_t size() const;

		const std::string& name(size_t i) const;
		expression::base* exp(size_t i);

		int getNameIndex(const std::string& name) const;

	private:

		friend class program;
		void add(const std::string& name, expression::base* e);

	private:

		std::vector<std::string> m_names;
		pointer_vector<expression::base> m_exps;
	};

	//
	// Program
	class program
	{
	public:

		program(stream::input<lexer::token>& i);
		~program();

		definitions& defs();
		expression::base* exp();

	private:

		definitions m_defs;
		expression::base* m_exp;
	};
}

#endif
