#include "syntacter_lambda.h"
#include "../../__common/binaries.h"
#include "../expression/expression_lambda.h"

#include <stdexcept>

//////////////////////////////////////////////////
//
// Public functions
//
///////////////////////////////////////////////////

expression::base* syntacter::read_lambda(stream::input<lexer::token>& input)
{
	return expression::lambda::read(input);
}

//////////////////////////////////////////////////
//
// Private functions
//
///////////////////////////////////////////////////

expression::lambda* expression::lambda::read(stream::input<lexer::token>& input)
{
	std::auto_ptr<lambda> le(new lambda());

	lexer::token t = input.get();

	// Argument list

	if (t.type() == lexer::token::open_bracket)
	{
		// ( arg* )
		// ( arg+ . arg)

		for (int i = 0;;i++)
		{
			t = input.get();

			if (t.type() == lexer::token::close_bracket)
			{
				// End of arguments list
				break;
			}

			if (t.type() == lexer::token::variable) 
			{
				// Usual argument
				le->addArgument(t.getString());
			}
			else if (t.type() == lexer::token::point)
			{
				// Point
				
				if (i == 0)
				{
					// args must be non empty
					throw std::runtime_error("expression::lambda::read: non arguments before point");
				}

				t = input.get();

				if (!t.isVariable())
				{
					// one argument must follow point
					throw std::runtime_error("expression::lambda::read: non arguments after point");
				}

				le->setListArgument(t.getString());

				t = input.get();

				if (t.type() != lexer::token::close_bracket)
				{
					// only one argument!
					throw std::runtime_error("expression::lambda::read: too more arguments after point");
				}

				input.putback(t);
			}
			else
			{
				throw std::runtime_error("expression::lambda::read: waiting for argument list");
			}
		}		
	}
	else if (t.isVariable())
	{
		// One list argument
		le->setListArgument(t.getString());
	}
	else 
	{
		throw std::runtime_error("expression::lambda::read: waiting for argument list");
	}

	// Expression
	le->setFunction(syntacter::read(input));
	
	// )
	t = input.get();

	if (t.type() != lexer::token::close_bracket)
	{
		throw std::runtime_error("expression::lambda::read: waiting for )");
	}

	return le.release();
}