#include "syntacter_literal.h"
#include "../expression/expression_literals.h"

#include <stdexcept>

namespace syntacter
{
	expression::base* read_list(stream::input<lexer::token>& i);
}

//////////////////////////////////////////////////
//
// Public functions
//
///////////////////////////////////////////////////

//
// Read literal from stream
expression::base* syntacter::read_literal(stream::input<lexer::token>& i)
{
	lexer::token t = i.get();

	switch (t.type())
	{
	case lexer::token::quote:
		return read_literal(i);
	case lexer::token::boolean:
		return new expression::boolean(t.getBoolean());
	case lexer::token::character:
		return new expression::character(t.getCharacter());
	case lexer::token::string:
		return new expression::string(t.getString());
	case lexer::token::numeric:
		return new expression::numeric(t.getNumeric());
	}

	if (t.isIdentifier())
	{
		return new expression::symbol(t.getString());
	}

	if (t.type() == lexer::token::open_bracket)
	{
		// List
		return read_list(i);
	}

	throw std::runtime_error("syntacter:read_literal: Unknown literal");
}

///////////////////////////////////////////////////
//
// Private functions
//
///////////////////////////////////////////////////

expression::base* syntacter::read_list(stream::input<lexer::token>& i)
{
	lexer::token t = i.get();

	if (t.type() == lexer::token::close_bracket)
	{
		// Empty list
		return new expression::empty_list();
	}

	i.putback(t);

	// Read car
	std::auto_ptr<expression::base> car(read_literal(i));

	t = i.get();

	if (t.type() == lexer::token::point)
	{
		// (car . cdr)
		std::auto_ptr<expression::base> cdr(read_literal(i));
		t = i.get();

		if (t.type() == lexer::token::close_bracket)
		{
			return new expression::pair(car.release(), cdr.release());
		}
		else
		{
			throw std::runtime_error("syntacter:read_list: waiting for close bracket after .cdr");
		}
	}
	else
	{
		i.putback(t);
		return new expression::pair(car.release(), read_list(i));
	}
}
