#ifndef dump_utils_smartptr_h
#define dump_utils_smartptr_h

#pragma once

//
// Smart pointer for something
// with methods addref and release
template<typename T>
class smartptr
{
public:

	smartptr(const T* p = 0) : m_p(p)
	{
		if (m_p) m_p->addref();
	}

	smartptr(const smartptr<T>& p)
	{
		m_p = p.m_p;
		if (m_p) m_p->addref();
	}

	smartptr& operator=(const smartptr<T>& p)
	{
		if (&p != this)
		{
			if (m_p) m_p->release();
			m_p = p.m_p;
			if (m_p) m_p->addref();
		}

		return *this;
	}

	~smartptr()
	{
		if (m_p) m_p->release();
	}

	const T* operator->() const
	{
		return m_p;
	}

	const T* get() const
	{
		return m_p;
	}

private:

	const T* m_p;
};

#endif
