#include "loader.h"
#include "../../__common/binaries.h"

#include <stdexcept>

/////////////////////////////
//
// record
//
/////////////////////////////

loader::record::record(const char* base /*= 0*/, const binary::record* rec /*= 0*/) 
		: m_base(base), m_rec(rec)
{
}

short loader::record::type() const
{
	return m_rec->type;
}

short loader::record::flags() const
{
	return m_rec->flags;
}

int loader::record::estimate() const
{
	return m_rec->estimate;
}

const char* loader::record::data() const
{
	return m_base + m_rec->offset;
}

size_t loader::record::size() const
{
	return m_rec->size;
}

/////////////////////////////////////////////
//
// executable
//
////////////////////////////////////////////

loader::executable::executable(const char* name) : m_mmf(name)
{
	// file must contain header
	if (m_mmf.size() < sizeof(binary::header))
	{
		throw std::runtime_error("loader::executable: file is not correct, m_mmf.size() < sizeof(binary::header)");
	}

	const binary::header* ph = (const binary::header*)m_mmf.get();

	// right header
	if (ph->signature != binary::signature)
	{
		throw std::runtime_error("loader::executable: file is not correct, wrong signature");
	}

	// right header
	if (ph->version != binary::version)
	{
		throw std::runtime_error("loader::executable: file is not correct, wrong version");
	}

	// file must contain table
	if (m_mmf.size() < ph->objectTableOffset + ph->objectTableBytesSize)
	{
		throw std::runtime_error("loader::executable: file is not correct, m_mmf.size() < table");
	}

	// table of records
	if (ph->objectTableBytesSize % sizeof(binary::record) != 0)
	{
		throw std::runtime_error("loader::executable: file is not correct, ph->objectTableBytesSize % sizeof(binary::record) != 0");
	}

	m_data.resize(ph->objectTableBytesSize / sizeof(binary::record));
	const binary::record* rec = (const binary::record*)(m_mmf.get() + ph->objectTableOffset);

	// for each record
	{for (size_t i = 0; i < m_data.size(); i++, rec++)
	{
		// file must contain record data
		if (rec->offset + rec->size > m_mmf.size())
		{
			throw std::runtime_error("loader::executable: file is not correct, m_mmf.size() < recort");
		}

		m_data[i] = record(m_mmf.get(), rec);
	}}
}

size_t loader::executable::size() const
{
	return m_data.size();
}

const loader::record& loader::executable::operator[](size_t i) const
{
	return m_data[i];
}

