#ifndef executer_object_object_literals_h
#define executer_object_object_literals_h

#include "object_base.h"
#include "../../__common/binaries.h"
#include <string>

namespace object
{
	//
	// Numeric object
	class numeric : public base
	{
	public:

		static proxy create(double v);
		static proxy create(const char* v, size_t size);

		double value() const;

		virtual bool equiv(const base* o) const;
		virtual std::string toString() const;

	private:

		numeric(double v);

	private:

		double m_value;
	};

	//
	// Boolean object
	class boolean : public base
	{
	public:

		static proxy create(bool v);
		static proxy create(const char* v, size_t size);

		bool value() const;

		virtual bool equiv(const base* o) const;
		virtual bool isTrue() const;
		virtual std::string toString() const;

	private:

		boolean(bool v);

	private:

		bool m_value;
	};

	//
	// Character object
	class character : public base
	{
	public:

		static proxy create(char v);
		static proxy create(const char* v, size_t size);

		char value() const;

		virtual bool equiv(const base* o) const;
		virtual std::string toString() const;

	private:

		character(char v);

	private:

		char m_value;
	};

	//
	// String object
	class string : public base
	{
	public:

		static proxy create(const std::string& v);
		static proxy create(const char* v, size_t size);

		const std::string& value() const;

		virtual bool equiv(const base* o) const;
		virtual std::string toString() const;

	private:

		string(const char* v, size_t size);

	private:

		std::string m_value;
	};

	//
	// Symbol object
	class symbol : public base
	{
	public:

		static proxy create(const std::string& v);
		static proxy create(const char* v, size_t size);

		virtual bool equiv(const base* o) const;
		virtual std::string toString() const;

	private:

		symbol(const char* v, size_t size);

	private:

		std::string m_value;
	};

	//
	// Pair object
	class pair : public base
	{
	public:

		static proxy create(const context::base* c, const char* v, size_t size);
		static proxy cons(proxy head, proxy tail);
		static proxy list(size_t s, const proxy* item);
		
		proxy car() const;
		proxy cdr() const;
		bool isList() const;

		virtual proxy evaluate(bool separable) const;
		virtual std::string toString() const;

	private:

		std::string toListString() const;

		pair(const context::base* c, binary::link head, binary::link tail);
		pair(proxy head, proxy tail);

	private:

		binary::link m_carLink;
		binary::link m_cdrLink;

		mutable proxy m_car;
		mutable proxy m_cdr;
	};

	//
	// Empty_list object
	class empty_list : public base
	{
	public:

		static proxy create(const char* v, size_t size);
		static proxy create();

		virtual bool equiv(const base* o) const;
		virtual std::string toString() const;

	private:

		empty_list();		
	};
}
#endif
